# -*- coding: utf-8 -*-

from __future__ import absolute_import

import sys
import logging
import json
import argparse

from etl.sources import get_source
from etl.transform import transform_source
from etl.destinations import get_destination
from etl.process import get_process
from etl.token import get_token_provider

try:
    import colorlog
except ImportError:
    colorlog = None

logger = logging.getLogger('etl')


class Parser(object):
    def __init__(self):
        self.parser = argparse.ArgumentParser()
        self.parser.add_argument(
            '-v',
            '--verbose',
            action='store_true',
            default=False,
        )
        self.parser.add_argument(
            '-n',
            '--dry-run',
            default=False,
            action='store_true',
            dest='dryrun',
            help='Dont actually push the values',
        )

    def parse_args(self, *args):
        return self.parser.parse_args(*args)

    def with_infile(self):
        self.parser.add_argument(
            'filename',
            metavar='file.(csv|xlsx|json)',
            help='The input CSV file',
        )
        self.parser.add_argument(
            '--json',
            dest='filetype',
            action='store_const',
            default=None,
            const='application/json',
        )
        self.parser.add_argument(
            '--csv',
            dest='filetype',
            action='store_const',
            const='text/csv',
        )
        self.parser.add_argument(
            '--xlsx',
            dest='filetype',
            action='store_const',
            const='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
        )
        self.parser.add_argument(
            '-H',
            '--no-header',
            default=True,
            dest='has_header',
            action='store_false',
            help='Indicates that the content starts at the first line',
        )
        self.parser.add_argument(
            '-O',
            '--offset',
            default=-1,
            type=int,
            help='Skip a number of records',
        )
        self.parser.add_argument(
            '-l',
            '--limit',
            default=-1,
            type=int,
            help='Limit the number of records to process',
        )
        return self

    def with_map_parse(self):
        self.parser.add_argument(
            '-r',
            '--record',
            action='append',
            dest='record_fn',
            metavar='FUNCTION',
            default=[],
            help='Apply the function to the record',
        )
        self.parser.add_argument(
            '-m',
            '--map',
            type=lambda x: x.split('='),
            metavar='source_key=destination_key',
            action='append',
            default=[],
            help='maps source_key to destination_key. Can be specified more than once',
        )
        self.parser.add_argument(
            '-y',
            '--copy',
            type=lambda x: x.split('='),
            metavar='duplicated_key=original_key',
            dest='duplicate',
            default=[],
            action='append',
        )
        self.parser.add_argument(
            '-d',
            '--delete',
            dest='remove',
            default=[],
            action='append',
        )
        self.parser.add_argument(
            '-p',
            '--parse',
            type=lambda x: x.split('='),
            metavar='key=type',
            default=[],
            action='append',
            help='parse key using the type function. Can be specified more than once. example: bool, int, float',
        )
        return self

    def with_command_parser(self):
        self.parser.add_argument(
            '-k',
            '--ignore-ssl-cert',
            action='store_true',
            dest='insecure_ssl',
            default=False,
        )

        self.parser.add_argument(
            'target',
            metavar='URL',
            help='The url of the web service',
        )
        self.parser.add_argument(
            '-c',
            '--column',
            default=[],
            dest='columns',
            action='append',
            help='Name the columns, should be repeated as many time as the number of columns',
        )
        self.parser.add_argument(
            '-T',
            '--token',
            help='The key used to authenticate the requests',
            default=None,
        )
        self.parser.add_argument(
            '-t',
            '--system-tokens',
            action='store_true',
            help='Use the system token manager',
            default=None,
        )
        self.parser.add_argument(
            '-P',
            '--parallel',
            type=int,
            help='Enable the parallel treatment',
        )
        return self

    def with_meta_file(self):
        self.parser.add_argument(
            'file',
            metavar='META',
            help='A json file containing a map of the options of the process',
        )
        return self


class BaseMain(object):

    def setup_logging(self, verbose):
        logger = logging.getLogger()
        if colorlog:
            formatter = colorlog.ColoredFormatter('%(bold_blue)s%(asctime)10s%(reset)s '
                                                  '[%(log_color)s%(levelname)8s%(reset)s]'
                                                  '%(bold_purple)s%(name)25s%(reset)s %(message)s')
        else:
            formatter = logging.Formatter(u'[%(levelname)8s] %(name)16s: %(message)s')

        sh = logging.StreamHandler()
        sh.setFormatter(formatter)
        sh.setLevel(logging.DEBUG)

        logger.setLevel(logging.DEBUG if verbose else logging.INFO)
        logger.addHandler(sh)

        logging.getLogger('requests.packages.urllib3.connectionpool').setLevel(
            logging.INFO if verbose else logging.WARNING
        )

    def __call__(self, filename, **kw):
        self.setup_logging(kw.get('verbose', False))

        token_provider = get_token_provider(kw.get('system_tokens'), kw.get('token'))
        logger.debug('Getting tokens from %s', token_provider)

        source = get_source(filename, kw.get('filetype'), token_provider)
        logger.debug('Source is %s', source)

        destination = get_destination(kw.get('target'), token_provider, kw.get('dryrun'), kw.get('insecure_ssl'))
        logger.debug('Destination is %s', destination)

        transformed_source = transform_source(source, kw, *kw.get('stages', []),
                                              has_header=kw.get('has_header'),
                                              columns=kw.get('columns'))
        logger.debug('Transormed source is %s', transformed_source)

        process = get_process(transformed_source, destination, kw.get('parallel', 1))
        logger.debug('Process is %s', process)
        return process()


class ExitCatchMixin(object):
    def __call__(self, *args, **kw):
        try:
            r = super(ExitCatchMixin, self).__call__(*args, **kw)
        except Exception:
            import traceback
            traceback.print_exc()
            sys.exit(1)
        else:
            if r is True or r is False:
                sys.exit(1 - r)


class FileMain(ExitCatchMixin, BaseMain):
    def __init__(self, parser):
        self.parser = parser

    def __call__(self, args):
        options = self.parser.parse_args(args)
        try:
            with open(options.file, 'r') as source:
                kw = json.load(source)
        except IOError as e:
            sys.stderr.write('Cannot open {}: {}\n'.format(options.file, e))
            sys.exit(1)
        except ValueError as e:
            sys.stderr.write('Cannot parse {}: {}\n'.format(options.file, e))
            sys.exit(1)
        else:
            kw.update(vars(options))
            super(FileMain, self).__call__(**kw)


class CommandMain(ExitCatchMixin, BaseMain):
    def __init__(self, parser, **defaults):
        super(CommandMain, self).__init__()
        self.parser = parser
        self.defaults = defaults

    def __call__(self, args):
        options = self.parser.parse_args(args)
        kw = self.defaults.copy()
        kw.update(vars(options))
        super(CommandMain, self).__call__(**kw)


main = CommandMain(Parser().with_command_parser().with_map_parse().with_infile())


if __name__ == '__main__':
    main(None)
