# -*- coding: utf-8 -*-

import logging

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

from etl.databases.backends.base import Backend

logger = logging.getLogger(__name__)


class DatabaseConnections(object):
    def __init__(self, backend, token_provider):
        self._connections = {}
        assert isinstance(backend, Backend), type(backend)
        self.backend = backend
        self.token_provider = token_provider

    def __repr__(self):
        return repr(self.backend)

    def close(self):
        for c in self._connections.values():
            c.close()
        self._connections.clear()

    def _connect_to(self, host, port, user, db, charset):
        key = (user, host, port, db)
        if key not in self._connections:
            logger.info('Opening connection to %s@%s:%s/%s', *key)
            password = None if self.backend.without_password else self.token_provider(host=host, username=user)
            self._connections[key] = self.backend.open_connection(host=host, user=user, port=port,
                                                                  password=password, db=db, charset=charset)
        return self._connections[key]

    def get_connection(self, url):
        url_parsed = urlparse(url)
        if not url_parsed.fragment:
            raise ValueError('destination url {} does not have a fragment'.format(url))

        query_dict = dict((query.split('=', 1) if '=' in query else (query, True))
                          for query in url_parsed.query.split('&'))
        connection = self._connect_to(url_parsed.hostname, url_parsed.port,
                                      url_parsed.username, url_parsed.path.lstrip('/'),
                                      query_dict.get('charset', 'utf-8'))
        return (connection, url_parsed.fragment)

    def build_insert_query(self, url, record):
        conn, table = self.get_connection(url)
        sql_query, params = self.backend.build_insert_query(table, record)
        return PreparedRequest(conn, sql_query, params)


class PreparedRequest(object):
    def __init__(self, conn, query, params):
        self._conn = conn
        self.query = query
        self.params = params

    def __repr__(self):
        return '{} ({})'.format(self.query, self.params)

    def __eq__(self, other):
        return (isinstance(other, PreparedRequest) and
                self._conn is other._conn and
                self.query == other.query and
                self.params == other.params
                )

    def __call__(self):
        cursor = self._conn.cursor()
        try:
            cursor.execute(self.query, self.params)
        except Exception:
            self._conn.rollback()
            raise
        else:
            self._conn.commit()
        finally:
            cursor.close()
