# -*- coding: utf-8 -*-

import json
import os
import logging

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse


logger = logging.getLogger(__name__)


class YAMLFormatter(object):
    def __init__(self):
        import yaml
        self._yaml = yaml

    def __call__(self, record):
        return self._yaml.dump(record)


class JSONFormatter(object):
    def __call__(self, record):
        return json.dumps(record)


class FileDestination(object):
    @classmethod
    def from_scheme(cls, target, scheme, ctx):
        if scheme != 'file':
            return None

        if 'yaml' in ctx.fragment:
            formatter = YAMLFormatter()
        elif 'json' in ctx.fragment:
            formatter = JSONFormatter()
        elif ctx.ext == '.yaml' or ctx.ext == '.yml':
            formatter = YAMLFormatter()
        elif ctx.ext == '.json':
            formatter = JSONFormatter()
        else:
            formatter = None

        return cls(target, formatter)

    def __init__(self, target, formatter):
        self._target = target
        self._files = {}
        self._formatter = formatter or JSONFormatter()

    def __repr__(self):
        return '<File {}>'.format(self._target)

    def __enter__(self):
        return self

    def __exit__(self, exc_value, exc_type, traceback):
        self._close_files()

    def _close_files(self):
        for file in self._files.values():
            try:
                file.close()
            except Exception:
                logger.exception('Cannot close %s', file)
        self._files.clear()

    def _open(self, filename):
        if filename not in self._files:
            parent_dir = os.path.dirname(filename)
            if not os.path.isdir(parent_dir):
                os.makedirs(parent_dir)
            logger.debug('Open %s', filename)
            self._files[filename] = open(filename, 'w')
        return self._files[filename]

    def _open_file(self, url):
        url_parsed = urlparse(url)
        return self._open(url_parsed.path)

    def __call__(self, record):
        url = self._target(record)
        file = self._open_file(url)
        file.write(self._formatter(record))
