# -*- coding: utf-8 -*-

import requests
import json
import datetime
from decimal import Decimal
import logging

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

from requests.auth import AuthBase

logger = logging.getLogger(__name__)


class TokenAuthentication(AuthBase):

    def __init__(self, token):
        # setup any auth-related data here
        assert callable(token), token
        self.token = token

    def __call__(self, r):
        token = self.token(host=urlparse(r.url).netloc, username='')
        authorization_header = 'Token {}'.format(token)
        # modify and return the request
        r.headers['Authorization'] = authorization_header
        return r


class RequestMaker(object):
    def __init__(self, auth=None, insecure_ssl=False):
        self.auth = auth
        self.verify_ssl = not insecure_ssl
        self._session = requests.Session()

    def __repr__(self):
        return '{}/{}'.format('anon' if self.auth is None else 'auth',
                              'SSL' if self.verify_ssl else 'SSL?')

    def __call__(self, method, url, data, **kw):
        logger.debug('Request %s %s', method, url)
        try:
            resp = self._session.request(
                method, url,
                allow_redirects=False,
                auth=self.auth,
                data=data,
                verify=self.verify_ssl,
                **kw
            )
        except requests.RequestException as e:
            logger.error('Record [%s] failed, %s', repr(data)[:20], e)
            return
        if not (200 <= resp.status_code < 400):
            logger.error('Record [%s] failed, %s', repr(data)[:20], resp)
            logger.warning('Failure was %s', resp.text[:1000])
        else:
            logger.debug('Resp %s', resp)


class fakefloat(float):
    def __init__(self, decimal):
        self.decimal = decimal

    def __repr__(self):
        return str(self.decimal)


class DecimalJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, (datetime.date, datetime.datetime)):
            return o.isoformat()
        if isinstance(o, Decimal):
            return fakefloat(o)
        return super(DecimalJSONEncoder, self).default(o)


def json_decimal_encoder(record):
    return 'application/json', json.dumps(record, cls=DecimalJSONEncoder)


def json_encoder(record):
    return 'application/json', json.dumps(record)


def form_encoded_encoder(record):
    return None, record


class HTTPDestination(object):
    @classmethod
    def from_scheme(cls, target, scheme, ctx):
        if scheme not in {'http', 'https'}:
            return None

        auth = TokenAuthentication(ctx.token) if ctx.token else None
        if ctx.insecure_ssl:
            import requests
            requests.packages.urllib3.disable_warnings()
        if 'PUT' in ctx.fragment:
            method = 'PUT'
        else:
            method = 'POST'

        if 'jsondecimal' in ctx.fragment:
            encoder = json_decimal_encoder
        elif 'json' in ctx.fragment:
            encoder = json_encoder
        else:
            encoder = None

        rmaker = RequestMaker(auth=auth, insecure_ssl=ctx.insecure_ssl)
        return cls(rmaker, target, method=method, encoder=encoder)

    def __init__(self, rmaker, target, method='POST', encoder=None):
        self.rmaker = rmaker
        self.method = method
        self._target = target
        self._encoder = encoder or form_encoded_encoder

    def __repr__(self):
        return '<{} {} {}>'.format(self.method, self.rmaker, self._target)

    def __call__(self, record):
        content_type, data = self._encoder(record)
        headers = {
            'X-requested-with': 'XMLHttpRequest'
        }
        if content_type:
            headers['Content-Type'] = content_type

        return self.rmaker(
            method=self.method,
            url=self._target(record),
            headers=headers,
            data=data,
        )
