# -*- coding: utf-8 -*-

import logging
import collections

from .stream import StreamTransformer, LimitSource, OffsetSource, Filter, Skip
from .record import ListMapper, KeyRenamer, KeyRemover, KeyParser, KeyDuplicate
from .callbacks import record_callbacks

logger = logging.getLogger(__name__)


PreprocessStage = collections.namedtuple('PreprocessStage', ['columns', 'has_header'])
Stage = collections.namedtuple('Stage', ['offset', 'limit', 'skip', 'filter', 'map',
                                         'duplicate', 'remove', 'parse', 'record_fn'])


def transform_source(source, *stages, **kw):
    preprocess_stage = PreprocessStage(kw.pop('columns', None), kw.pop('has_header', False))
    assert not kw, 'remaining keywords %r' % kw

    source = preprocess(source, preprocess_stage)
    for stage in stages:
        stage = Stage(
            offset=stage.get('offset', 0),
            limit=stage.get('limit', -1),
            map=stage.get('map'),
            duplicate=stage.get('duplicate'),
            remove=stage.get('remove'),
            parse=stage.get('parse'),
            skip=stage.get('skip'),
            filter=stage.get('filter'),
            record_fn=stage.get('record_fn'),
        )
        source = _tranform_source(source, stage)
    return source


def preprocess(source, stage):
    source = iter(source)
    if stage.has_header:
        header_row = next(source)
        assert isinstance(header_row, list), 'First record is not a list'
    else:
        header_row = []

    if stage.columns:
        logger.debug('Appy column map %s', stage.columns)
        return StreamTransformer(source, ListMapper(stage.columns))
    elif header_row:
        logger.debug('Appy header row %s', header_row)
        return StreamTransformer(source, ListMapper(header_row))
    else:
        return source


def _tranform_source(source, stage):
    source = preprocess_source(source, stage)
    transformations = get_transformations(stage)
    if transformations:
        logger.debug('Applying transformations: %s', transformations)
        source = StreamTransformer(source, *transformations)
    return source


def preprocess_source(source, stage):
    if stage.offset > 0:
        source = OffsetSource(source, stage.offset)
    if stage.limit > 0:
        source = LimitSource(source, stage.limit)
    if stage.skip:
        source = Skip(source, *stage.skip)
    if stage.filter:
        source = Filter(source, *stage.filter)
    return source


def get_transformations(stage):
    transformations = []
    if stage.map:
        transformations.append(KeyRenamer(stage.map))
    if stage.duplicate:
        transformations.append(KeyDuplicate(dict(stage.duplicate)))
    if stage.remove:
        transformations.append(KeyRemover(stage.remove))
    if stage.parse:
        if isinstance(stage.parse, collections.Mapping):
            key_parsers = stage.parse
        else:
            key_parsers = collections.defaultdict(list)
            for k, v in stage.parse:
                key_parsers[k].append(v)
        transformations.append(KeyParser(key_parsers))
    if stage.record_fn:
        transformations.extend(record_callbacks.get_callbacks(stage.record_fn))
    return transformations
