# -*- coding: utf-8 -*-

import functools
try:
    from urllib.parse import quote, unquote
except ImportError:
    from urlparse import unquote
    from urllib import quote


class CallbackRegistry(object):
    def __init__(self, callback_map=None):
        self.callback_map = dict(callback_map) if callback_map else {}

    def get_callbacks(self, callback_names):
        if isinstance(callback_names, str) or callable(callback_names):
            callback_names = [callback_names]
        return [self.get_callback(name) for name in callback_names]

    def get_callback(self, callback_name):
        if callable(callback_name):
            return callback_name

        if '?' in callback_name:
            callback_name, args_spec = callback_name.split('?', 1)
            args, kwargs = [], {}
            for a in args_spec.split('&'):
                if '=' in a:
                    k, v = a.split('=', 1)
                    kwargs[unquote(k)] = unquote(v)
                else:
                    args.append(unquote(a))
        else:
            args = kwargs = None

        callback_fn = self.get_callback_by_name(callback_name)
        if args is not None:
            return callback_fn(*args, **kwargs)
        return callback_fn

    def get_callback_by_name(self, callback_name):
        try:
            return self.callback_map[callback_name]
        except KeyError:
            raise ValueError('Unknown callback {}'.format(callback_name))


def bool_(value):
    return value.lower() == 'true'


def unwrap(value):
    return value.strip('\'"')


def strip_html(value):
    from htmllaundry import sanitize
    return sanitize(value)


def iso8601_render(value):
    return value.isoformat()


def listize(value):
    return [value]


def slugify(value):
    return value.replace(' ', '-').lower().replace('/', '-').strip('-')


def urlencode(value):
    return quote(value, '')


class coalesce(object):
    def __init__(self, default):
        self.default = default

    def __call__(self, value):
        return self.default if value is None else value


class remove(object):
    def __init__(self, *values):
        self.values = values

    def __call__(self, value):
        return functools.reduce(lambda value, to_remove: value.replace(to_remove, ''), self.values, value)

    @property
    def __name__(self):
        return 'remove({})'.format(', '.join(self.values))


item_callbacks = CallbackRegistry({
    'unwrap': unwrap,
    bool: bool_,
    'bool': bool_,
    'int': int,
    'float': float,
    'strip_html': strip_html,
    'iso8601_render': iso8601_render,
    'listize': listize,
    'slugify': slugify,
    'remove': remove,
    'coalesce': coalesce,
    'urlencode': urlencode,
})


def has_key(key):
    @functools.wraps(has_key)
    def inner_has_key(record):
        return key in record
    return inner_has_key


def not_empty(key):
    @functools.wraps(not_empty)
    def inner_has_key(record):
        return bool(record.get(key))
    return inner_has_key


class dictize(object):
    def __init__(self, target, *key_clone, **keys):
        self.target = target
        keys.update((k, k) for k in key_clone)
        self.keys = keys

    def __repr__(self):
        return '[{}]={{{}}}'.format(self.target,
                                    ', '.join('{}={}'.format(k, v) for k, v in self.keys.items()))

    def __call__(self, record):
        record[self.target] = {target_key: record.pop(key) for target_key, key in self.keys.items()}
        return record


class listize_record(object):
    def __init__(self, target, *keys):
        self.target = target
        self.keys = keys

    def __repr__(self):
        return '[{}]=[{}]'.format(self.target, ', '.join(self.keys))

    def __call__(self, record):
        record[self.target] = [record.pop(key) for key in self.keys]
        return record


record_callbacks = CallbackRegistry({
    'has_key': has_key,
    'not_empty': not_empty,
    'dictize': dictize,
    'listize': listize_record,
})
