# -*- coding: utf-8 -*-


import functools

from etl.transform.callbacks import item_callbacks


class EmptyValue(object):
    """Set an empty values for each given key"""

    def __init__(self, keys):
        self.keys = frozenset(keys)

    def __repr__(self):
        return ', '.join('{}=:empty'.format(k) for k in self.keys)

    def __call__(self, record):
        record.update({
            k: '' for k in self.keys
        })
        return record


class PruneEmptyKeys(object):
    """Remove empty value from record if they are in given keys"""

    def __init__(self, keys):
        self.keys = frozenset(keys)

    def __repr__(self):
        return ','.join('{}?'.format(k) for k in self.keys)

    def __call__(self, record):
        return {
            k: v
            for k, v in record.items()
            # Copy if value si defined or key not in list of keys to prune
            if v or k not in self.keys
        }


class KeyRemover(object):
    """Remove all given keys"""

    def __init__(self, keys):
        self.keys = frozenset(keys)

    def __repr__(self):
        return ', '.join('!{}'.format(k) for k in self.keys)

    def __call__(self, record):
        return {
            k: v
            for k, v in record.items()
            if k not in self.keys
        }


class KeyRenamer(object):
    """Rename keys according to the map"""

    def __init__(self, map_):
        self.map = dict(map_)

    def __repr__(self):
        return ', '.join('{}->{}'.format(k, v) for k, v in self.map.items())

    def __call__(self, record):
        return {self.map.get(k, k): v for k, v in record.items()}


class KeyDuplicate(object):
    """Duplicate the value from a key to a mapped key"""

    def __init__(self, map_):
        self.map = dict(map_)

    def __repr__(self):
        return ', '.join('{}->{}'.format(v, k) for k, v in self.map.items())

    def __call__(self, record):
        for k, v in self.map.items():
            record[k] = record[v]
        return record


class ListMapper(object):
    """Transform a list to a dict with given keys"""

    def __init__(self, keys):
        self.keys = keys

    def __repr__(self):
        return '({!r})'.format(self.keys)

    def __call__(self, record):
        return dict(zip(self.keys, record))


class KeyParser(object):
    """Apply parser to keys in the record"""

    def __init__(self, parser_names):
        self.parsers = {k: item_callbacks.get_callbacks(vs) for k, vs in parser_names.items()}

    def __repr__(self):
        return ', '.join('{}->{}()'.format(k, '()->'.join(v.__name__ for v in vs))
                         for k, vs in self.parsers.items())

    def __call__(self, record):
        return {k: self.parse(k, v) for k, v in record.items()}

    def parse(self, k, v):
        if k not in self.parsers:
            return v
        return functools.reduce(lambda prec, fn: fn(prec), self.parsers[k], v)
