# -*- coding: utf-8 -*-

import logging

from etl.transform.callbacks import record_callbacks

logger = logging.getLogger(__name__)


class OffsetSource(object):
    """Discard all records from the source until the given offset"""
    def __init__(self, source, offset):
        self._source = source
        self.offset = offset

    def __repr__(self):
        return '{!r}[{}:]'.format(self._source, self.offset)

    def __iter__(self):
        iterator = iter(self._source)
        try:
            for x in range(self.offset):
                next(iterator)
        except StopIteration:
            logger.warning('Exhausted iterator at %s without stepping to %s', x, self.offset)
            return
        for record in iterator:
            yield record


class LimitSource(object):
    """Discard all records from the source after the given limit"""
    def __init__(self, source, limit):
        self._source = source
        self.limit = limit

    def __repr__(self):
        if isinstance(self._source, OffsetSource):
            return '{!r}[{}:{}]'.format(self._source._source, self._source.offset, self._source.offset + self.limit)
        return '{!r}[:{}]'.format(self._source, self.limit)

    def __iter__(self):
        for i, x in zip(range(self.limit), self._source):
            yield x


class CallbacksApplication(object):
    def __init__(self, source, *callbacks):
        self.source = source
        self.callbacks = callbacks

    def __repr__(self):
        return '{}({}, {})'.format(
            type(self).__name__[:3],
            self.source,
            ', '.join(getattr(callback, '__name__', repr(callback))
                      for callback in self.callbacks))


class StreamTransformer(CallbacksApplication):
    """Apply callbacks to each record of the stream"""

    def __iter__(self):
        for record in self.source:
            for callback in self.callbacks:
                record = callback(record)
            yield record


class Filter(CallbacksApplication):
    """Filter records matching some function"""
    def __init__(self, source, *callbacks):
        super(Filter, self).__init__(source, *record_callbacks.get_callbacks(callbacks))

    def __iter__(self):
        for record in self.source:
            if all(c(record) for c in self.callbacks):
                yield record


class Skip(CallbacksApplication):
    """Skip records matching some function"""

    def __init__(self, source, *callbacks):
        super(Skip, self).__init__(source, *record_callbacks.get_callbacks(callbacks))

    def __iter__(self):
        for record in self.source:
            if not any(c(record) for c in self.callbacks):
                yield record
